#ifndef PAGING_H
#define PAGING_H

#include "os.h"

#define PHYSICAL_MEMORY 0x10000000 // 256MB
#define PAGESIZE        0x1000     // 0x1000 = 4096 = 4K

// bitset algorithms.
#define INDEX_FROM_BIT(a)   (a/32)
#define OFFSET_FROM_BIT(a)  (a%32)

ULONG placement_address = (ULONG) 0x1000000; // 16 MB

// A bitset of frames - used or free
ULONG*  frames;
ULONG  nframes;

/************************************************************************/
// page_directory ==> page_table ==> page
typedef struct page
{
    ULONG present    :  1;   // Page present in memory
    ULONG rw         :  1;   // 0: read-only, 1: read/write
    ULONG user       :  1;   // 0: Supervisor level
    ULONG accessed   :  1;   // Has the page been accessed since last refresh?
    ULONG dirty      :  1;   // Has the page been written to since last refresh?
    ULONG unused     :  7;   // Combination of unused and reserved bits
    ULONG frame      : 20;   // Frame address (shifted right 12 bits)
} page_t;

typedef struct page_table
{
    page_t pages[1024];
} page_table_t;

typedef struct page_directory
{
    page_table_t* tables[1024];
    ULONG tablesPhysical[1024];
    ULONG physicalAddr;
} page_directory_t;
/************************************************************************/

void              paging_install();
//void              switch_page_directory(page_directory_t* new);
page_t*           get_page(ULONG address, int make, page_directory_t* dir);
page_directory_t* clone_directory(page_directory_t* src);

void              alloc_frame(page_t *page, int is_kernel, int is_writeable);
void              free_frame (page_t *page);

ULONG             kmalloc_int(ULONG sz, int align, ULONG* phys);
ULONG             kmalloc_a(ULONG sz);
ULONG             kmalloc_ap(ULONG sz, ULONG* phys);
ULONG             kmalloc(ULONG sz);

extern void copy_page_physical(ULONG source_address, ULONG dest_address); //process.asm

#endif
